@@ -1,19 +1,100 @@
 package Bot::Cobalt::Plugin::Alarmclock;
-$Bot::Cobalt::Plugin::Alarmclock::VERSION = '0.019003';
+$Bot::Cobalt::Plugin::Alarmclock::VERSION = '0.020001';
 use strictures 2;
 use v5.10;
 
+use File::Spec ();
+
 use Bot::Cobalt;
+use Bot::Cobalt::DB;
 use Bot::Cobalt::Utils 'timestr_to_secs';
 
 use Object::Pluggable::Constants ':ALL';
 
-# $self->{$timerid} = [ $context, $username ]
-sub new { bless +{}, shift }
+
+sub new { 
+  bless +{ 
+    # $self->timers->{$timerid} = [ $context, $username ]
+    _timers => +{},
+    _db     => undef,
+  }, shift
+}
+
+sub timers        { shift->{_timers} }
+sub clear_timers  { shift->{_timers} = +{} }
+
+sub _init_from_db {
+  my ($self) = @_;
+  my $db = $self->{_db};
+  unless ($db->dbopen) {
+    logger->error("dbopen failure for alarmclock db in _init_from_db");
+    logger->error("persistent alarms may be broken!");
+    return
+  }
+  my $count = 0;
+  ID: for my $id ($db->dbkeys) {
+    my $alarm = $db->get($id);
+    unless ($alarm && ref $alarm eq 'HASH') {
+      logger->warn(
+        defined $alarm ? 
+            "Alarm '$id' not a HASH; alarmclock db may be broken!"
+          : "Could not retrieve alarm '$id'; alarmclock db may be broken!"
+      );
+      next ID
+    }
+    my $expires_at = $alarm->{At};
+    unless ($expires_at) {
+      logger->warn("No 'At' expiry for timer id '$id', removing");
+      $db->del($id);
+      next ID
+    }
+    if ($expires_at <= time) {
+      logger->debug("Expiring stale alarmclock '$id'");
+      $db->del($id);
+      next ID
+    }
+    $alarm->{Alias} = plugin_alias($self);
+    my $secs = $expires_at - time;
+    my $new_id = core->timer_set( $secs, $alarm );
+    if ($new_id) {
+      $self->timers->{$new_id} = [ $alarm->{Context}, $alarm->{User} ];
+      $db->put($new_id => $alarm);
+      $db->del($id);
+      ++$count
+    } else {
+      logger->warn("Failed to readd alarmclock timer '$id'");
+    }
+  }
+  $db->dbclose;
+  $count
+}
+
+sub _delete_alarm {
+  my ($self, $id) = @_;
+  my $db = $self->{_db};
+  unless ($db->dbopen) {
+    logger->error("dbopen failure for alarmclock db in _delete_alarm");
+    logger->error("persistent alarms may be broken!");
+  }
+  my $ret = $db->del($id);
+  unless ($ret) {
+    logger->warn("attempted to delete nonexistant alarm ID '$id'");
+  }
+  $db->dbclose;
+  $ret
+}
+
 
 sub Cobalt_register {
   my ($self, $core) = splice @_, 0, 2;
 
+  # FIXME config option for persistent alarms?
+  my $dbpath = File::Spec->catfile( $core->var, 'alarmclock.db' );
+  $self->{_db} = Bot::Cobalt::DB->new(
+    file => $dbpath,
+  );
+  my $count = $self->_init_from_db;
+
   register( $self, SERVER => qw/
     public_cmd_alarmclock
     public_cmd_alarmdelete
@@ -21,19 +102,17 @@ sub Cobalt_register {
     executed_timer
   / );
 
-  logger->info("Loaded alarm clock");
+  logger->info("Loaded alarm clock ($count existing alarms loaded)");
 
-  return PLUGIN_EAT_NONE
+  PLUGIN_EAT_NONE
 }
 
 sub Cobalt_unregister {
   my ($self, $core) = splice @_, 0, 2;
-
   logger->info("Unregistering core IRC plugin");
-
   core->timer_del_alias( core->get_plugin_alias($self) );
-
-  return PLUGIN_EAT_NONE
+  $self->clear_timers;
+  PLUGIN_EAT_NONE
 }
 
 sub Bot_deleted_timer { Bot_executed_timer(@_) }
@@ -43,14 +122,15 @@ sub Bot_executed_timer {
   my $timerid = ${$_[0]};
 
   return PLUGIN_EAT_NONE
-    unless exists $self->{$timerid};
+    unless exists $self->timers->{$timerid};
 
   logger->debug("clearing timer state for $timerid")
     if core->debug > 1;
 
-  delete $self->{$timerid};
+  delete $self->timers->{$timerid};
+  $self->_delete_alarm($timerid);
 
-  return PLUGIN_EAT_NONE
+  PLUGIN_EAT_NONE
 }
 
 sub Bot_public_cmd_alarmdelete { Bot_public_cmd_alarmdel(@_) }
@@ -70,7 +150,7 @@ sub Bot_public_cmd_alarmdel {
 
   my $channel = $msg->channel;
 
-  unless (exists $self->{$timerid}) {
+  unless (exists $self->timers->{$timerid}) {
     broadcast( 'message', $context, $channel,
       core->rpl( q{ALARMCLOCK_NOSUCH},
         nick    => $nick,
@@ -81,14 +161,12 @@ sub Bot_public_cmd_alarmdel {
     return PLUGIN_EAT_ALL
   }
 
-  my $thistimer = $self->{$timerid};
-
+  my $thistimer = $self->timers->{$timerid};
   my ($ctxt_set, $ctxt_by) = @$thistimer;
-
-  ## ... did this user set this timer?
-  unless ($ctxt_set eq $context && $auth_usr eq $ctxt_by) {
+  ## did this user set this timer?
+  ## original user may've been undef if LevelRequired == 0
+  unless ($ctxt_set eq $context && defined $ctxt_by && $auth_usr eq $ctxt_by) {
     my $auth_lev = core->auth->level($context, $nick);
-
     ## superusers can override:
     unless ($auth_lev == 9999) {
       broadcast( 'message', $context, $channel,
@@ -97,13 +175,13 @@ sub Bot_public_cmd_alarmdel {
           timerid => $timerid,
         )
       );
-
       return PLUGIN_EAT_ALL
     }
   }
 
   core->timer_del($timerid);
-  delete $self->{$timerid};
+  delete $self->timers->{$timerid};
+  $self->_delete_alarm($timerid);
 
   broadcast( 'message', $context, $channel,
     core->rpl( q{ALARMCLOCK_DELETED},
@@ -112,7 +190,7 @@ sub Bot_public_cmd_alarmdel {
     )
   );
 
-  return PLUGIN_EAT_ALL
+  PLUGIN_EAT_ALL
 }
 
 
@@ -128,9 +206,14 @@ sub Bot_public_cmd_alarmclock {
   my $minlevel = $cfg->{LevelRequired} // 1;
 
   ## quietly do nothing for unauthorized users
-  return PLUGIN_EAT_NONE
-    unless core->auth->level($context, $setter) >= $minlevel;
-
+  unless (core->auth->level($context, $setter) >= $minlevel) {
+    logger->debug(
+      "ignoring unauthorized alarmclock from '$setter' on '$context'"
+    );
+    return PLUGIN_EAT_NONE
+  }
+  
+  ## undef if $minlevel == 0
   my $auth_usr = core->auth->username($context, $setter);
 
   ## This is the array of (format-stripped) args to the _public_cmd_
@@ -146,25 +229,33 @@ sub Bot_public_cmd_alarmclock {
   my $secs = timestr_to_secs($timestr) || 1;
   my $channel = $msg->channel;
 
-  my $id = core->timer_set( $secs,
-    {
-      Type => 'msg',
-      Context => $context,
-      Target => $channel,
-      Text   => $txtstr,
-      Alias  => plugin_alias($self),
-    }
-  );
+  my $alarm = +{
+    Type    => 'msg',
+    User    => $auth_usr,
+    Context => $context,
+    Target  => $channel,
+    Text    => $txtstr,
+    Alias   => plugin_alias($self),
+    At      => time + $secs,
+  };
+  my $id = core->timer_set( $secs, $alarm );
 
   my $resp;
   if ($id) {
-    $self->{$id} = [ $context, $auth_usr ];
+    $self->timers->{$id} = [ $context, $auth_usr ];
     $resp = core->rpl( q{ALARMCLOCK_SET},
         nick => $setter,
         secs => $secs,
         timerid => $id,
         timestr => $timestr,
     );
+    my $db = $self->{_db};
+    if ($db->dbopen) {
+      $db->put($id => $alarm);
+      $db->dbclose;
+    } else {
+      logger->error("dbopen failure for alarmclock db during add");
+    }
   } else {
     $resp = core->rpl( q{RPL_TIMER_ERR} );
   }
@@ -173,7 +264,7 @@ sub Bot_public_cmd_alarmclock {
     broadcast( 'message', $context, $channel, $resp );
   }
 
-  return PLUGIN_EAT_ALL
+  PLUGIN_EAT_ALL
 }
 
 1;
@@ -187,6 +278,7 @@ Bot::Cobalt::Plugin::Alarmclock - Timed IRC highlights
 
 =head1 SYNOPSIS
 
+  # On IRC:
   !alarmclock 20m go do some something
   !alarmclock 1h30m stop staring at irc
 
@@ -205,7 +297,7 @@ For example:
 
 (Accuracy down to the second is not guaranteed. Plus, this is IRC. Sorry.)
 
-Mimics B<darkbot6> behavior, but with vaguely sane time string grammar.
+As of C<v0.20.1>, alarmclocks will persist between bot runs.
 
 =head1 AUTHOR
 
