@@ -1,5 +1,5 @@
 package Bot::Cobalt::Plugin::Extras::Karma;
-$Bot::Cobalt::Plugin::Extras::Karma::VERSION = '0.019003';
+$Bot::Cobalt::Plugin::Extras::Karma::VERSION = '0.020001';
 ## simple karma++/-- tracking
 
 use Carp;
@@ -27,7 +27,7 @@ sub Cobalt_register {
   my $dbpath = File::Spec->catfile( $core->var, 'karma.db' );
   
   $self->{karmadb} = Bot::Cobalt::DB->new(
-    File => $dbpath,
+    file => $dbpath,
   );
 
   $self->{karma_regex} = qr/^(\S+)(\+{2}|\-{2})$/;
